<?php

namespace Samwilson\PhpFlickr;

class PhotosetsApi extends ApiMethodGroup
{

    /**
     * Add a photo to the end of an existing photoset.
     * @link https://www.flickr.com/services/api/flickr.photosets.addPhoto.html
     * @param int $photosetId The ID of the photoset to add a photo to.
     * @param int $photoId
     * @return bool
     */
    public function addPhoto($photosetId, $photoId)
    {
        $args = [
            'photoset_id' => $photosetId,
            'photo_id' => $photoId,
        ];
        $response = $this->flickr->request('flickr.photosets.addPhoto', $args, true);
        return (bool)$response;
    }

    /**
     * Create a new photoset for the calling user.
     * This method requires authentication with 'write' permission.
     * New photosets are automatically put first in the photoset ordering for the user.
     * @see PhotosetsApi::orderSets() if you don't want the new set to appear first on the user's
     * photoset list.
     * @link https://www.flickr.com/services/api/flickr.photosets.create.html
     * @param string $title A title for the photoset.
     * @param string $description A description of the photoset. May contain limited HTML.
     * @param int $primaryPhotoId The ID of the photo to represent this set. The photo must belong
     * to the calling user.
     * @return bool
     */
    public function create($title, $description, $primaryPhotoId)
    {
        $args = [
            'title' => $title,
            'primary_photo_id' => $primaryPhotoId,
            'description' => $description,
        ];
        $response = $this->flickr->request('flickr.photosets.create', $args, true);
        return isset($response['photoset']) ? $response['photoset'] : false;
    }

    /**
     * Delete a photoset. This method requires authentication with 'write' permission.
     * @link https://www.flickr.com/services/api/flickr.photosets.delete.html
     * @param int $photosetId The id of the photoset to delete. It must be owned by the calling user.
     * @return bool
     */
    public function delete($photosetId)
    {
        $args = ['photoset_id' => $photosetId];
        return (bool)$this->flickr->request('flickr.photosets.delete', $args, true);
    }

    /**
     * Modify the meta-data for a photoset.
     * @link https://www.flickr.com/services/api/flickr.photosets.editMeta.html
     * @param int $photosetId The ID of the photoset to modify.
     * @param string $title The new title for the photoset.
     * @param string|null $description A description of the photoset. May contain limited HTML.
     * @return bool
     */
    public function editMeta($photosetId, $title, $description = null)
    {
        $args = [
            'photoset_id' => $photosetId,
            'title' => $title,
            'description' => $description,
        ];
        return (bool)$this->flickr->request('flickr.photosets.editMeta', $args, true);
    }

    /**
     * Modify the photos in a photoset. Use this method to add, remove and re-order photos.
     * @link https://www.flickr.com/services/api/flickr.photosets.editPhotos.html
     * @param int $photosetId The ID of the photoset to modify. The photoset must belong to the
     * calling user.
     * @param int $primaryPhotoId The ID of the photo to use as the 'primary' photo for the set.
     * This ID must also be passed along in $photoIds parameter.
     * @param string|string[] $photoIds An array or comma-delimited list of photo IDs to include in
     * the set. They will appear in the set in the order sent. This list must contain the primary
     * photo ID. All photos must belong to the owner of the set. This list of photos replaces the
     * existing list. Call flickr.photosets.addPhoto to append a photo to a set.
     * @return bool
     */
    public function editPhotos($photosetId, $primaryPhotoId, $photoIds)
    {
        if (is_array($photoIds)) {
            $photoIds = join(',', $photoIds);
        }
        $args = [
            'photoset_id' => $photosetId,
            'primary_photo_id' => $primaryPhotoId,
            'photo_ids' => $photoIds,
        ];
        return (bool)$this->flickr->request('flickr.photosets.editPhotos', $args, true);
    }

    /**
     * Returns next and previous photos for a photo in a set.
     * @link https://www.flickr.com/services/api/flickr.photosets.getContext.html
     * @param int $photoId The ID of the photo to fetch the context for.
     * @param int $photosetId The ID of the photoset for which to fetch the photo's context.
     * @return mixed[] Array with 'prevphoto' and 'nextphoto' keys.
     */
    public function getContext($photoId, $photosetId)
    {
        $args = [
            'photo_id' => $photoId,
            'photoset_id' => $photosetId,
        ];
        return $this->flickr->request('flickr.photosets.getContext', $args);
    }

    /**
     * Gets information about a photoset.
     * @link https://www.flickr.com/services/api/flickr.photosets.getInfo.html
     * @param int $photosetId The ID of the photoset to fetch information for.
     * @param string $userId The ID of the owner of the set passed in $photosetId.
     * @return mixed[]|bool
     */
    public function getInfo($photosetId, $userId)
    {
        $args = [
            'photoset_id' => $photosetId,
            'user_id' => $userId,
        ];
        $response = $this->flickr->request('flickr.photosets.getInfo', $args);
        return isset($response['photoset']) ? $response['photoset'] : false;
    }

    /**
     * Returns the photosets belonging to the specified user.
     * @link https://www.flickr.com/services/api/flickr.photosets.getList.html
     * @param string $userId The NSID of the user to get a photoset list for. If none is
     * specified, the calling user is assumed.
     * @param int $page The page of results to get. Currently, if this is not provided, all sets are
     * returned, but this behaviour may change in future.
     * @param int $perPage The number of sets to get per page. If paging is enabled, the maximum
     * number of sets per page is 500.
     * @param string $primaryPhotoExtras A comma-delimited list of extra information to fetch for
     * the primary photo. Currently supported fields are: license, date_upload, date_taken,
     * owner_name, icon_server, original_format, last_update, geo, tags, machine_tags, o_dims,
     * views, media, path_alias, url_sq, url_t, url_s, url_m, url_o
     * @param string $photoIds A comma-separated list of photo ids. If specified, each returned set
     * will include a list of these photo IDs that are present in the set as
     * "has_requested_photos".
     * @return mixed[]|bool
     */
    public function getList(
        $userId = null,
        $page = null,
        $perPage = null,
        $primaryPhotoExtras = null,
        $photoIds = null
    ) {
        $args = [
            'user_id' => $userId,
            'page' => $page,
            'per_page' => $perPage,
            'primary_photo_extras' => $primaryPhotoExtras,
            'photo_ids' => $photoIds,
        ];
        $response = $this->flickr->request('flickr.photosets.getList', $args);
        return isset($response['photosets']) ? $response['photosets'] : false;
    }

    //flickr.photosets.getPhotos
    //flickr.photosets.orderSets
    //flickr.photosets.removePhoto
    //flickr.photosets.removePhotos
    //flickr.photosets.reorderPhotos
    //flickr.photosets.setPrimaryPhoto
}
